#-------------------------#
# ncluster = 1            #
#-------------------------#






OneCluster<-function(longdata=NULL,outcomedata=NULL, nknot=4, mcsim, nburnin,ncluster, p.null, p.target, interim=TRUE,
                    stop.input=NULL){
  
  library(HI)       # to use arms function
  library(mvnfast)  # to use rmvn and dmvn function to draw samples
  library(gtools)   # to use rdirichlet function
 
  #-------------------------------------#
  # A. Longitudinal part data organize  #
  #-------------------------------------#
  data<-longdata
  
  #---------------------------------------------#
  # I. Spline part creation                     #
  #---------------------------------------------#
  spline.t1<-data[,3]
  spline.t2<-spline.t1^2
  
  pre.k<-seq(0,1,length.out=nknot+2)
  myknot<-pre.k[2:(length(pre.k)-1)]
  spline.f<-function(x,knot){
    myf<-NULL
    for (i in 1:length(knot)){
      myf[i]<-(ifelse(x>=knot[i],(x-knot[i])^2,0))
    }
    return(myf)
  }
  
  spline.t3<-t(sapply(spline.t1,spline.f,knot=myknot))  #dimension: nrow=ngroup*npatient*length(t), ncol=nknot
  z<-as.numeric(data[,1]) # dependent variable, vector format
  
  
  #-------------------------------------------------------------#
  # II. extract dimension parameters from the data:             #
  # number of observations of each patient (N_obs);             #
  # number of total servations(N_tot)= N_subj*length(t);        #
  # number of subjects in the data (N_subj);                    #
  # number of groups in the data (N_grp);                       #
  #-------------------------------------------------------------#
  N_obs<-as.numeric(as.vector(table(data[,2])))
  N_tot<-sum(N_obs)
  N_subj<-as.numeric(length(unique(data[,2])))
  N_grp<-as.numeric(length(unique(data[,4])))
  
  sub<-as.numeric(match(data[,2], unique(data[,2]))) # subject ID indicator
  grp<-as.numeric(match(data[,4], unique(data[,4]))) # group ID indicator
  #------------------------------------------------#
  # III. Vectors to store the posterior samples    #
  # of parameters and initialize them:             #
  # alpha: all the spline coefficients             #
  # beta: all the r.e., ui and wij                 #
  # tau_u: precision of u_i, group-specific r.e.   #
  # tau_w: precision of w_ij, subject-specfic r.e. #
  # tau_e: precision of e_ijt, measurement error   #
  # tau_phi: precision of phi_k, r.e. of spline    #
  #------------------------------------------------#
  
  alpha<-matrix(NA, nrow=mcsim, ncol=(3+nknot))
  beta<-matrix(NA, nrow=mcsim, ncol=N_grp+N_subj)
  tau_phi<-tau_u<-tau_w<-tau_e<-rep(NA,mcsim)
  betaY<-rep(NA,mcsim)
  # Initialize
  alpha[1, ]<- rep(0, (3+nknot))
  betaY[1]<-0
  beta[1,]<-rep(0,N_grp+N_subj)
  tau_phi[1]<-tau_u[1]<-tau_w[1]<-tau_e[1]<-1
 
  
  
  #Design matrix for longitudinal part
  X_spline <- cbind(rep(1, N_tot), spline.t1, spline.t2, spline.t3)
  X_u <-NULL
  for (i in 1: N_grp){
    grp.temp<-grp
    grp.temp[which(grp==i)]<-1
    grp.temp[which(grp!=i)]<-0
    X_u<-cbind(X_u,grp.temp)
  }
  X_w<-matrix(0, nrow=N_tot,ncol=N_subj)
  for (i in 1:N_subj){
    X_w[which(sub==i),i]<-1
  }
  X<-cbind(X_spline, X_u, X_w)
  X_random<-cbind(X_u, X_w)
  #------------------------------------------------------------------#
  #-------------------------------------#
  # B. binary part data organize        #
  #-------------------------------------#  
  ydata<-outcomedata
  y<-as.numeric(ydata[,1]) # outcome variable
  
  #----------------------------------------------#
  # I. extract dimension parameters from data    #
  # N_pt: number of patients per group; vector   #
  # N_grp_y: number of groups in outcome data    #
  # N_tot_y: number of total obs in outcome data #
  #----------------------------------------------#
  N_pt<-as.numeric(as.vector(table(ydata[,2])))
  N_grp_y<-as.numeric(length(unique(ydata[,2])))  
  N_tot_y<-sum(N_pt)
  
  grp_y<-as.numeric(match(ydata[,2], unique(ydata[,2])))# group ID indicator in outcome data
  
  
  X_y<-NULL
  for (i in 1:N_grp_y){
    X_y<-c(X_y, rep(y[grp_y==i],each=N_obs[1]))
  }
  #------------------------------------------------------#
  # II. Vectors to store the posterior samples           #
  # of parameters and initialize them:                   #
  # mi: the group-specific r.e.                          #
  # theta: the cluster-specific f.e.                     # 
  # tau_mi: precision of m_i(k)                          #
  # p.posterior: posterior of response rate by subgroup  #
  #------------------------------------------------------#
  theta<-rep(NA, mcsim)
  tau_mi<-rep(NA,mcsim)
  mi<-matrix(NA, nrow=mcsim, ncol=N_grp_y)
  p.posterior<-matrix(NA, nrow=mcsim, ncol=N_grp_y)
  
  mi[1,]<-rep(0, N_grp_y)
  tau_mi[1]<-2
  
  
  theta.global<-(log(p.null/(1-p.null))+log(p.target/(1-p.target)))/2
  theta[1]<-theta.global
  
  p.posterior[1,]<-exp(theta.global+mi[1,])/(1+exp(theta.global+mi[1,]))
  #------------------------------------------------------------------#  

  
  #----------------------------------------------#
  # D. MCMC part                                 #
  #                                              #
  #----------------------------------------------#    
  a<-0.001; b<-0.001
  a.spline<-10^(-3); b.spline<-10^(-3)
  a.mi<-0.001; b.mi<-0.001
  #DIC calculation
  logbinom<-numeric(N_grp_y)  
  loglongden<-numeric(N_grp) 
  
  logbinom.dev<-numeric(N_grp_y)
  loglongden.dev<-numeric(N_grp)
  loglike<-NULL
  #-----------------------------------------------------------#
  # log density of theta and mi in the outcome process,       #
  # will be used in arms                                      #
  #-----------------------------------------------------------#
  
  # log density of theta 
  logden.theta <- function(x, myy, mymi, theta.true) {
    temp <- x + mymi
    return(sum(myy*temp - log(1+exp(temp))) - (10^(-4))*0.5*(x-theta.true)^2)
  }
  
  
  
  # log density of random intercept mi
  # x: the random intercept mi
  logden.mi <- function(x, mytheta, myy, mytau_mi) {
    temp <- mytheta + x
    return(sum(myy * temp - log(1+exp(temp))) - 0.5*(x^2)*mytau_mi)
  }
  
  #log density of tau
  logden.tau<- function(x,mymi, a.mi, b.mi,ngrp){
    return((1+a.mi+ngrp/2)*log(x)-x*(b.mi+0.5*sum(mymi^2)))
  }
  
  #---------------------------------#
  
  set.seed(100+q)
  for (iter in 2:mcsim){
    
    
    #----------------------#  
    # update outcome part  #
    #----------------------#
    
    # update mi
    for (i in 1:N_grp_y){
      mi[iter, i] <- arms(mi[iter-1, i], logden.mi, function(x,...) (x>(-5))*(x<5), 1,
                          mytheta=theta[iter-1], myy=y[grp_y==i], mytau_mi=tau_mi[iter-1])
    }
    
    # update theta
    # update tau_mi
    
    mi.temp<-mi[iter,]
    tau_mi[iter] <-rgamma(1, a.mi+(N_grp_y/2), scale=(b.mi+0.5*sum(mi[iter,]^2))^(-1) )    
      
    mi.long<-rep(mi[iter,],N_pt )
    theta[iter]<- arms(theta[iter-1], logden.theta, function(x,...) (x>(-5))*(x<(5)), 1, myy=y, mymi=mi.long, theta.true=theta.global)
      
    
    
    
    #update p.posterior
    
      p.posterior[iter,]<-exp(theta[iter]+mi[iter,])/(1+exp(theta[iter]+mi[iter,]))
  
    
    
    
    
    
    #---------------------------#  
    # update longitudinal part  #
    #---------------------------#   
    
    # update alpha 
    # update tau_phi
   
    sigma.spline<-diag(c(rep(10^4, 3), rep(1/tau_phi[iter-1], nknot)), nrow=3+nknot, ncol=3+nknot)
    sigma.alpha<-solve(tau_e[iter-1]*crossprod(X_spline)+solve(sigma.spline))
    mean.alpha<-tau_e[iter-1]*sigma.alpha%*%crossprod(X_spline, (z-X_random%*%matrix(beta[iter-1,],ncol=1)-X_y*betaY[iter-1]))
    alpha[iter, ] <-rmvn(1, mean.alpha, sigma.alpha)
    alpha_spline.temp<-alpha[iter, (4:(3+nknot))]
    tau_phi[iter]<-rgamma(1, a.spline+0.5*nknot, scale=(b.spline+0.5*sum(alpha_spline.temp^2))^(-1))
    

    # update beta
    # update tau_e  #
    res_sum<-0
    for (i in 1: N_grp){
      sigma.random<-diag(c(1/tau_u[iter-1], rep(1/tau_w[iter-1], N_pt[i])), nrow=1+N_pt[i], ncol=1+N_pt[i])
      idx.temp<-c(i, (sum(N_pt[1:i])+N_grp-(N_pt[i]-1)):(N_grp+sum(N_pt[1:i])))
      sigma.beta<-solve(solve(sigma.random)+crossprod(X_random[grp==i,idx.temp])*tau_e[iter-1])
      mu.beta<-tau_e[iter-1]*sigma.beta%*%crossprod(X_random[grp==i,idx.temp],(z[grp==i]-X_spline[grp==i,]%*%alpha[iter,]-X_y[grp==i]*betaY[iter-1]))
      
      beta[iter,idx.temp]<-rmvn(1, mu.beta, sigma.beta)
      
      
      res_temp<-z[grp==i]-X_spline[grp==i,]%*%alpha[iter,]-X_random[grp==i,idx.temp]%*%beta[iter,idx.temp]-X_y[grp==i]*betaY[iter-1]
      res_sum<-res_sum+sum(res_temp^2)

    }
    tau_e[iter]<-rgamma(1, a+0.5*N_tot, scale=(b+0.5*res_sum)^(-1))
    
    
    # update betaY
    var.betaY<-1/(10^(-4)+tau_e[iter]*crossprod(X_y))
    mean.betaY<-var.betaY*tau_e[iter]*crossprod(X_y, z-X_spline%*%alpha[iter,]-X_random%*%beta[iter,])
    betaY[iter]<-rnorm(1,mean.betaY, sqrt(var.betaY))
    
    
    # update tau_u#
    tau_u[iter]<-rgamma(1, a+0.5*N_grp, scale=(b+0.5*sum(beta[iter, (1:N_grp)]^2))^(-1))
    
    
    # update tau_w #
    tau_w[iter]<-rgamma(1, a+0.5*N_subj, scale=(b+0.5*sum(beta[iter, ((N_grp+1):(N_grp+N_subj))]^2))^(-1))
    
    
    #Posterior Deviance mean calculation 
    if (iter >=(nburnin+1)){
      for (i in 1:N_grp){
        # loglikelihood for DIC calculation #
        
        prob.res<- exp(theta[iter]+mi[iter, i])/(1+ exp(theta[iter]+mi[iter, i]))
        mi.var<-(tau_mi[iter])^(-1)
        logbinom[i]<-dbinom(sum(y[grp_y==i]),N_pt[i], prob.res, log=TRUE)
        #+dnorm(mi[iter,i], 0, sqrt(mi.var), log=TRUE)
        
        idx.temp<-c(i, (sum(N_pt[1:i])+N_grp-(N_pt[i]-1)):(N_grp+sum(N_pt[1:i])))
        sig.long <- (tau_e[iter]^(-1)*diag(nrow=N_pt[i]*N_obs[1], ncol=N_pt[i]*N_obs[1]))
        mean.long <- X_spline[grp==i,]%*%alpha[iter, ]+X_random[grp==i,idx.temp]%*%beta[iter,idx.temp]+X_y[grp==i]*betaY[iter]
        loglongden[i]<-dmvn(z[grp==i], mean.long, sig.long, log=TRUE)
  
      }
      loglike<- c(loglike, -2*sum(loglongden+logbinom))
      
    }
    
    
  }#end MCMC
  
  
  
  
  
  #----------------------------------------------#
  # E. DIC part                                  #
  # loglike: to store the loglikelihood          #
  #----------------------------------------------#   
  
  
  
  #------------------------#  
  
  
  
  dev.bar<-mean(loglike)
  
  # Deviance with posterior mean of parameters
  theta.mean<-mean(theta[(nburnin+1):mcsim])
  mi.mean<-apply(mi[(nburnin+1):mcsim,],2, mean)
  mivar.mean<-(mean(tau_mi[(nburnin+1):mcsim]))^(-1)
  tau_e.mean<-mean(tau_e[(nburnin+1):mcsim])
  alpha.update<-alpha[(nburnin+1):mcsim, ]
  beta.mean<-apply(beta[(nburnin+1):mcsim,],2, mean)
  tau_u.mean<-mean(tau_u[(nburnin+1):mcsim])
  tau_w.mean<-mean(tau_w[(nburnin+1):mcsim])
  tau_phi.mean<-mean(tau_phi[(nburnin+1):mcsim])
  betaY.est<-mean(betaY[(nburnin+1):mcsim])
  
  for (i in 1:N_grp){
    prob.postmean<-exp(theta.mean +mi.mean[i])/(1+exp(theta.mean+mi.mean[i]))
    logbinom.dev[i]<-dbinom(sum(y[grp_y==i]),N_pt[i], prob.postmean, log=TRUE)
      #dnorm(mi.mean[i], 0, sqrt(mivar.mean), log=TRUE)
    
    idx.temp<-c(i, (sum(N_pt[1:i])+N_grp-(N_pt[i]-1)):(N_grp+sum(N_pt[1:i])))
    sig.longmean <- (tau_e.mean^(-1)*diag(nrow=N_pt[i]*N_obs[1], ncol=N_pt[i]*N_obs[1]))
    mu.longmean <- X_spline[grp==i,]%*%colMeans(alpha.update)+X_random[grp==i,idx.temp]%*%beta.mean[idx.temp]+X_y[grp==i]*betaY.est
    
    loglongden.dev[i]<-dmvn(z[grp==i], mu.longmean, sig.longmean, log=TRUE)
      
  }
  dev<--2*sum(logbinom.dev+loglongden.dev)
  
  dic<-2*(dev.bar-dev)+dev
  
  
  #--------------------------------#
  # F. transform the alpha         #
  #--------------------------------#   
  
  alpha.ave<-c(colMeans(alpha.update), numeric(3+nknot))
  theta.Kmean<-c(theta.mean,0)
  tau_mi.save<-tau_mi[((nburnin+1):mcsim)]
  
  
  
  #--------------------------------#
  # G.Interim/Final analysis       #
  #--------------------------------#  
  futstop<-0.05
  eff.ref<-c(0.911,0.911,0.91,0.909,0.909, 0.909) 
  instruction<-"continue"
  eff.prob<-numeric(N_grp)
  p.post.mean<-numeric(N_grp)
  if (interim == TRUE){
    
    fut.prob<-sapply(seq(1,N_grp_y,1), function(x) mean(p.posterior[((nburnin+1):mcsim),x]>(p.null+p.target)/2))    # calculate the futility probability
    stop.input[which(fut.prob<futstop)]<-1                                                       # stop if the probability <0.05
    
    if (!(0 %in% stop.input)){                                  #if at this interim analysis, all group have stopped for futility
      eff.prob<-sapply(seq(1,N_grp_y,1), function(x) mean(p.posterior[((nburnin+1):mcsim),x]>p.null))
      trteff.indicator<-(eff.prob>eff.ref)*1                                                # summarize the efficacy arms
      p.post.mean<-apply(p.posterior[(nburnin+1):mcsim,],2,mean)
      instruction<-"interim-allstop"
      
    }
    
  } else {  # Final analysis
    
    eff.prob<-sapply(seq(1,N_grp_y,1), function(x) mean(p.posterior[((nburnin+1):mcsim),x]>p.null))
    trteff.indicator<-(eff.prob>eff.ref)*1                                                # summarize the efficacy arms
    p.post.mean<-apply(p.posterior[(nburnin+1):mcsim,],2,mean)
    instruction<-"end"
    
  }
  
  results<-list(trteff.prob=eff.prob, outcome.p.mean = p.post.mean, DIC=dic, 
                instruction=instruction, stop.output=stop.input,  alpha.post=alpha.ave, betaY=betaY.est,theta.Kmean=theta.Kmean, tau_mi=tau_mi.save)
  results
  
}#end function
